/*
 *  linux/arch/arm/mach-clps711x/time.c
 *
 *  Copyright (C) 2001 Deep Blue Solutions Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/sched.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/hardware/clps7111.h>
#include <asm/hardware/ep7212.h>

extern unsigned long (*gettimeoffset)(void);

/*
 * gettimeoffset() returns time since last timer tick, in usecs.
 *
 * 'LATCH' is hwclock ticks (see CLOCK_TICK_RATE in timex.h) per jiffy.
 *
 *
 *  actually, LATCH == ((CLOCK_TICK_RATE + HZ/2) / HZ)
 *
 *
 * 'tick' is usecs per jiffy.
 */
static unsigned long clps711x_gettimeoffset(void)
{
	unsigned long hwticks;
	hwticks = LATCH - (clps_readl(TC2D) & 0xffff);	/* since last underflow */
	return (hwticks * tick) / LATCH;
}

void __init clps711x_setup_timer(void)
{
	unsigned int syscon1;

	gettimeoffset = clps711x_gettimeoffset;

	printk("clps711x_setup_timer() SYSCON1 = 0x%08x adding TC2S and TC2M bits\n", 
			(unsigned) IO_SYSCON1);

	syscon1 = clps_readl(SYSCON1);
	syscon1 |= SYSCON1_TC2S | SYSCON1_TC2M;
	clps_writel(syscon1, SYSCON1);

	printk("clps711x_setup_timer() SYSCON1 = 0x%08x\n", (unsigned) IO_SYSCON1);
	printk("clps711x_setup_timer() SYSCON2 = 0x%08x\n", (unsigned) IO_SYSCON2);
	printk("clps711x_setup_timer() SYSCON3 = 0x%08x\n", (unsigned) IO_SYSCON3);

	clps_writel(LATCH-1, TC2D); /* 512kHz / 100Hz - 1 */

	xtime.tv_sec = clps_readl(RTCDR);
}


