/*
 *  PhatNoise Board abstraction
 *  Adds /proc/pn_board_ver and /dev/led
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>

#ifdef CONFIG_ARCH_PHATNOISE

#define BOARD_STRING "PhatNoise Board Unknown Version\n"

#ifdef CONFIG_ARCH_PHATNOISE10
#undef BOARD_STRING
#define BOARD_STRING "PhatNoise Board v1.0\n"
#endif

#ifdef CONFIG_ARCH_PHATNOISE11
#undef BOARD_STRING
#define BOARD_STRING "PhatNoise Board v1.1\n"
#endif

#define C_NONE          'D'
#define C_GREEN         'G'
#define C_RED           'R'
#define C_ORANGE        'O'

#define LED_MAJOR 13

#define SYS_OUTPUT_REG 0xff000003

#define SYS_BASE 0xff000000
#define UNIQUEID_BASE 0x2440
#define RANDOMID_BASE 0x2700
#define SYSFLG_BASE 0x0140

static char *board_rev = BOARD_STRING;
static char *banner __initdata = KERN_INFO BOARD_STRING;

static int led_major;

#ifdef CONFIG_PROC_FS
static int pn_board_read(char* page, char** start, off_t off, int count, int* eof, void* data)
{
  int len;
  len = sprintf(page,board_rev);
  return len;
}

static int pn_cpuid_read(char *page, char **start, off_t off, int count, int* eof, void* data)
{
  int len, i;
  unsigned char *memp=(unsigned char *)SYS_BASE;
  char retstr[128], tempstr[4], uniqueid[4], randomid[16], sysflg[4];
  memcpy(uniqueid,memp+UNIQUEID_BASE,4);
  memcpy(randomid,memp+RANDOMID_BASE,16);
  memcpy(sysflg,memp+SYSFLG_BASE,4);
  strcpy(retstr,"Unique ID: 0x");
  for (i=0; i<4; i++) {
    sprintf(tempstr,"%02x",uniqueid[i]);
    strcat(retstr,tempstr);
  }
  strcat(retstr,"\nRandom ID: 0x");
  for (i=0; i<16; i++) {
    sprintf(tempstr,"%02x",randomid[i]);
    strcat(retstr,tempstr);
  }
  strcat(retstr,"\nCPU: ");
  if (sysflg[3] & 0x20)
    strcat(retstr,"EP73xx, ver 0x");
  else
    strcat(retstr,"Unknown, ver 0x");
  sprintf(tempstr,"%02x",sysflg[3]>>6);
  strcat(retstr,tempstr);
  strcat(retstr,"\n");
  len = sprintf(page,"%s",retstr);
  return len;
}
#endif

static ssize_t led_write(struct file *file,
                         const char *buffer, size_t count, loff_t * loff)
{
  unsigned char *memp=(unsigned char *)SYS_OUTPUT_REG;
  char color;
  copy_from_user((void*)(&color),(const void*)buffer,1);
  *memp &= ~0x01;
  if (color == C_GREEN) *memp |= 0x01;
  return 1;
}

static ssize_t led_read(struct file *file,
                         char *buffer, size_t count, loff_t * loff)
{
  unsigned char *memp=(unsigned char *)SYS_OUTPUT_REG;
  char color=C_NONE;

  if (*memp & 0x01) color=C_GREEN;
  copy_to_user(buffer,(const char*)&color,1);
  return 1;
}

static int led_open(struct inode *inode, struct file *file)
{
  return 0;
}

static int led_release(struct inode *inode, struct file *file)
{
  return 0;
}

struct file_operations led_fops = {
	write:		led_write,
	read:		led_read,
	open:		led_open,
	release:	led_release,
};

static int __init pn_board_init(void)
{
  printk(banner);
#ifdef CONFIG_PROC_FS
  create_proc_read_entry("pn_board_ver",0,0,pn_board_read,NULL);
  create_proc_read_entry("pn_cpu_id",0,0,pn_cpuid_read,NULL);
#endif
  led_major = register_chrdev(LED_MAJOR,"led",&led_fops);
  if (led_major) {
    printk(KERN_WARNING "LED: unable to get major %d\n", LED_MAJOR);
  }
  else {
    led_major = LED_MAJOR;
    printk(KERN_INFO "LED: major %d\n", led_major);
  }
  return 0;
}

module_init(pn_board_init);

#endif /* CONFIG_ARCH_PHATNOISE */
