/*
 *  linux/include/asm-arm/arch-clps711x/hardware.h
 *
 *  This file contains the hardware definitions of the Prospector P720T.
 *
 *  Copyright (C) 2000 Deep Blue Solutions Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#include <linux/config.h>

#define CLPS7111_VIRT_BASE	0xff000000
#define CLPS7111_BASE		CLPS7111_VIRT_BASE

#define IO_BASE	CLPS7111_BASE

/*
 * The physical addresses that the external chip select signals map to is
 * dependent on the setting of the nMEDCHG signal on EP7211 and EP7212
 * processors.  CONFIG_EP72XX_BOOT_ROM is only available if these
 * processors are in use.
 */
#ifndef CONFIG_EP72XX_ROM_BOOT
#define CS0_PHYS_BASE		(0x00000000)
#define CS1_PHYS_BASE		(0x10000000)
#define CS2_PHYS_BASE		(0x20000000)
#define CS3_PHYS_BASE		(0x30000000)
#define CS4_PHYS_BASE		(0x40000000)
#define CS5_PHYS_BASE		(0x50000000)
#define CS6_PHYS_BASE		(0x60000000)
#define CS7_PHYS_BASE		(0x70000000)
#else
#define CS0_PHYS_BASE		(0x70000000)
#define CS1_PHYS_BASE		(0x60000000)
#define CS2_PHYS_BASE		(0x50000000)
#define CS3_PHYS_BASE		(0x40000000)
#define CS4_PHYS_BASE		(0x30000000)
#define CS5_PHYS_BASE		(0x20000000)
#define CS6_PHYS_BASE		(0x10000000)
#define CS7_PHYS_BASE		(0x00000000)
#endif

#if defined (CONFIG_ARCH_EP7211)

#define EP7211_VIRT_BASE	CLPS7111_VIRT_BASE
#define EP7211_BASE		CLPS7111_VIRT_BASE
#include <asm/hardware/ep7211.h>

#elif defined (CONFIG_ARCH_EP7212)

#define EP7212_VIRT_BASE	CLPS7111_VIRT_BASE
#define EP7212_BASE		CLPS7111_VIRT_BASE


#endif

#if defined (CONFIG_ARCH_EP7212)

#include <asm/hardware/ep7212.h>

#endif

#define SYSPLD_VIRT_BASE	0xfe000000
#define SYSPLD_BASE		SYSPLD_VIRT_BASE

#ifndef __ASSEMBLER__

#define PCIO_BASE		IO_BASE

#endif


#if  defined (CONFIG_ARCH_AUTCPU12)

#define  CS89712_VIRT_BASE	CLPS7111_VIRT_BASE
#define  CS89712_BASE		CLPS7111_VIRT_BASE

#include <asm/hardware/clps7111.h>
#include <asm/hardware/ep7212.h>
#include <asm/hardware/cs89712.h>

#endif


#if defined (CONFIG_ARCH_CDB89712)

#include <asm/hardware/clps7111.h>
#include <asm/hardware/ep7212.h>
#include <asm/hardware/cs89712.h>

/* dynamic ioremap() areas */
#define FLASH_START      0x00000000
#define FLASH_SIZE       0x800000
#define FLASH_WIDTH      4

#define SRAM_START       0x60000000
#define SRAM_SIZE        0xc000
#define SRAM_WIDTH       4

#define BOOTROM_START    0x70000000
#define BOOTROM_SIZE     0x80
#define BOOTROM_WIDTH    4


/* static cdb89712_map_io() areas */
#define REGISTER_START   0x80000000
#define REGISTER_SIZE    0x4000
#define REGISTER_BASE    0xff000000

#define ETHER_START      0x20000000
#define ETHER_SIZE       0x1000
#define ETHER_BASE       0xfe000000

#endif


#if defined (CONFIG_ARCH_EDB7211)

/*
 * The extra 8 lines of the keyboard matrix are wired to chip select 3 (nCS3) 
 * and repeat across it. This is the mapping for it.
 *
 * In jumpered boot mode, nCS3 is mapped to 0x4000000, not 0x3000000. This 
 * was cause for much consternation and headscratching. This should probably
 * be made a compile/run time kernel option.
 */
#define EP7211_PHYS_EXTKBD		CS3_PHYS_BASE	/* physical */

#define EP7211_VIRT_EXTKBD		(0xfd000000)	/* virtual */


/*
 * The CS8900A ethernet chip has its I/O registers wired to chip select 2 
 * (nCS2). This is the mapping for it.
 *
 * In jumpered boot mode, nCS2 is mapped to 0x5000000, not 0x2000000. This 
 * was cause for much consternation and headscratching. This should probably
 * be made a compile/run time kernel option.
 */
#define EP7211_PHYS_CS8900A		CS2_PHYS_BASE	/* physical */

#define EP7211_VIRT_CS8900A		(0xfc000000)	/* virtual */


/*
 * The two flash banks are wired to chip selects 0 and 1. This is the mapping
 * for them.
 *
 * nCS0 and nCS1 are at 0x70000000 and 0x60000000, respectively, when running
 * in jumpered boot mode.
 */
#define EP7211_PHYS_FLASH1		CS0_PHYS_BASE	/* physical */
#define EP7211_PHYS_FLASH2		CS1_PHYS_BASE	/* physical */

#define EP7211_VIRT_FLASH1		(0xfa000000)	/* virtual */
#define EP7211_VIRT_FLASH2		(0xfb000000)	/* virtual */

#endif /* CONFIG_ARCH_EDB7211 */


#ifdef CONFIG_ARCH_PHATNOISE


#define PHAT_PHYS_IDE			CS5_PHYS_BASE
#define PHAT_VIRT_IDE			(0xfe100000)

#endif




/*
 * Relevant bits in port D, which controls power to the various parts of
 * the LCD on the EDB7211.
 */
#define EDB_PD1_LCD_DC_DC_EN	(1<<1)
#define EDB_PD2_LCDEN		(1<<2)
#define EDB_PD3_LCDBL		(1<<3)



/*
 * Allow access to IO registers as if they were normal variables
 */

#define __IOB(x)        (*(volatile unsigned char *) (IO_BASE+(x)))
#define __IOL(x)        (*(volatile unsigned long *) (IO_BASE+(x)))

#define IO_PADR         __IOB(PADR)
#define IO_PBDR         __IOB(PBDR)
#define IO_PCDR         __IOB(PCDR)
#define IO_PDDR         __IOB(PDDR)
#define IO_PADDR        __IOB(PADDR)
#define IO_PBDDR        __IOB(PBDDR)
#define IO_PCDDR        __IOB(PCDDR)
#define IO_PDDDR        __IOB(PDDDR)
#define IO_PEDR         __IOB(PEDR)
#define IO_PEDDR        __IOB(PEDDR)
#define IO_SYSCON1      __IOL(SYSCON1)
#define IO_SYSCON2      __IOL(SYSCON2)
#define IO_SYSFLG       __IOL(SYSFLG)
#define IO_MEMCFG1      __IOL(MEMCFG1)
#define IO_MEMCFG2      __IOL(MEMCFG2)
/* DRFPR */
#define IO_INTSR1       __IOL(INTSR1)
#define IO_INTSR2       __IOL(INTSR2)
#define IO_INTSR3       __IOL(INTSR3)
#define IO_INTMR1       __IOL(INTMR1)
#define IO_INTMR2       __IOL(INTMR2)
#define IO_INTMR3       __IOL(INTMR3)
#define IO_LCDCON       __IOL(LCDCON)
#define IO_TC1D         __IOL(TC1D)
#define IO_TC2D         __IOL(TC2D)
#define IO_RTCDR        __IOL(RTCDR)
#define IO_RTCMR        __IOL(RTCMR)
/* PMPCON */
#define IO_CODR         __IOB(CODR)
#define IO_UARTDR       __IOL(UARTDR)
#define IO_UBRLCR       __IOL(UBRLCR)
#define IO_SYNCIO       __IOL(SYNCIO)
#define IO_PALLSW       __IOL(PALLSW)
#define IO_PALMSW       __IOL(PALMSW)
#define IO_STFCLR       __IOL(STFCLR)
#define IO_BLEOI        __IOL(BLEOI)
#define IO_MCEOI        __IOL(MCEOI)
#define IO_TEOI         __IOL(TEOI)
#define IO_TC1EOI       __IOL(TC1EOI)
#define IO_TC2EOI       __IOL(TC2EOI)
#define IO_RTCEOI       __IOL(RTCEOI)
#define IO_UMSEOI       __IOL(UMSEOI)
#define IO_COEOI        __IOL(COEOI)
#define IO_HALT         __IOL(HALT)
#define IO_STDBY        __TOL(STDBY)

/*-------------- For PhatNoise 7212 ------------------------*/
#define IO_SYSCON3      __IOL(SYSCON3)
#define IO_SYSFLG2      __IOL(SYSFLG2)

#define IO_DAIR         __IOL(DAIR)
#define IO_DAISR        __IOL(DAISR)
#define IO_DAIDR0       __IOL(DAIDR0)
#define IO_DAIDR1       __IOL(DAIDR1)
#define IO_DAIDR2       __IOL(DAIDR2)
#define IO_DAIFLG       __IOL(DAIFLG)

/* ----  Phatnoise 7312 ---- */

#define IO_DAI64FS	__IOL(DAI64FS)


#endif

